package starterCode;

import starterCode.NetParams;
import tests.NetChecker1;


/**
 * Class implementing a basic three-layer neural network.
 */
public class Net {

  /** The parameters. */
  public NetParams params;

  /** The checker that will verify the network's correctness. */
  public NetChecker1 checker;

  /** The bias unit, whose value is always 1. Not essential, but handy as a placeholder. */
  public Unit Bias;
  /** The first input unit. */
  public Unit inUnit1;
  /** The second input unit. */
  public Unit inUnit2;
  /** The output unit. */
  public Unit outUnit;

  /**
   * Constructor for Net class.
   * 
   * Create Net with the specified architecture and data.
   * 
   * @param patterns Array of input patterns (each an array of int)
   * @param targets Array of output patterns (each an array of int)
   */
  public Net(NetParams params,
      NetChecker1 checker) {
    this.params = params;
    this.checker = checker;

    // The Bias is implemented as a unit connected directly to every non-input node.
    // The bias node will always be active, and have no incoming connections.
    Bias = new Unit(params);
    Bias.activation = 1.0;

    // Make units
    inUnit1 = new Unit(params);
    inUnit2 = new Unit(params);
    outUnit = new Unit(params);

    // Make connections between units.
    inUnit1.addOutgoingUnit(outUnit);
    inUnit2.addOutgoingUnit(outUnit);
    outUnit.addIncomingUnit(inUnit1);
    outUnit.addIncomingUnit(inUnit2);
    outUnit.addIncomingUnit(Bias);
    inUnit1.initialize();
    inUnit2.initialize();
    outUnit.initialize();

    // Prepare to be checked
    if (this.checker != null)
      this.checker.initialize(this);
  }

  /**
   * feedforward()
   * 
   * @param pattern
   *          An input training pattern
   * 
   * Present pattern and compute activations for rest of net.
   */
  public void feedforward(int[] pattern) {
    // TODO: fill this in!
  }

  /**
   * computeError()
   * 
   * @return current network error
   * 
   * Present all patterns to network and calculate current error. Current error
   * is measured by sum of squared error on output nodes.
   */
  public double computeError() {
    // TODO: fill this in!
    return 0;
  }

  /**
   * Present target pattern and propagate error back.
   *
   * @param targets An output target pattern
   *
   */
  public void backprop(int[] target) {
    // TODO: fill this in!
  }

  /**
   * Update weights.
   *
   */
  public void updateStochasticWeights() {
    // TODO: fill this in!
  }
  
  public void accumulateWeightChange() {
    // TODO: fill this in!
  }
  
  public void updateAccumulatedWeights() {
    // TODO: fill this in!
  }
  
  /**
   * train()
   * 
   * Train the net according to the current training parameters for at most
   * the given number of epochs.
   * 
   * Initialize weights and present patterns in order, updating weights after
   * each pattern. Training continues until error falls below criterion, or
   * when numEpochs epochs have elapsed, whichever comes first.
   */
  public void train() {
    // TODO implement this! Don't forget to call the checker as follows:
    
    // for every training datum:
    //   checker.trainDatum(i);
    //   (do your computations here)
    //   checker.checkMyActivation(this);
    // if we're doing stochastic updates (at every datum), do this at every datum:
    //   checker.checkMyWeights(this);
    //   checker.checkMyMomentum(this);
    //   checker.checkMyWeightChange(this);
    // if we're doing batch mode updates (at every epoch), do this at every epoch:
    //   checker.checkMyWeights(this);
    //   checker.checkMyMomentum(this);
    //   checker.checkMyWeightChange(this);
  }

  /**
   * toString()
   * 
   * A method to neatly dump the weights of each link
   */
  public String toString() {
    String rStr = "\n";
    rStr += "Inputs: " + 2 + "\n";
    rStr += "No hidden layers\n";
    rStr += "Outputs: " + 1 + "\n";
    rStr += "Error: " + computeError() + "\n";
    rStr += "Weights: \n";

    rStr += "   " + "inUnit1" + " -> " + "outUnit" + " = " + outUnit.inWeights[0] + "\n";
    rStr += "   " + "inUnit2" + " -> " + "outUnit" + " = " + outUnit.inWeights[1] + "\n";
    rStr += "   " + "bias" + " -> " + "outUnit" + " = " + outUnit.inWeights[1] + "\n";
    return rStr;
  }

}
