package starterCode;

/**
 * 
 */

public class NetParams {
  public static enum TrainType { Stochastic, Batch }
  
  // Training data
  /** The training input data. */
  public int[][] trainingData;
  /** The training target data. */
  public int[][] trainingTargets;
  
  
  // Network training parameters
  /** The minimum weight for random initialization. */
  public double minWeight = -1;
  /** The maximum weight for random initialization. */
  public double maxWeight = 1;
  /** The network's learning rate. */
  public double learningRate = 0.1;
  /** The network's momentum factor. */
  public double momentum = 0.1;
  /** The network's error termination criterion. */
  public double errorCriterion = .1;
  /** The network's number of training epochs. */
  public int numEpochs = 100;
  /** The network's type of training. */
  public TrainType trainType = TrainType.Stochastic;

  // Network structure parameters: not used for Part 1
  /** The number of input signals to the network. */
  public int numInputs = 2;
  /** The number of outputs from the network. */
  public int numOutputs = 1;
  /** The number of hidden units per hidden layer. */
  public int numHidden = 0;
  /** The number of hidden layers. */
  public int numHidLayers = 0;
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("NetParams {\n");
    sb.append("  minWeight = " + minWeight + "\n");
    sb.append("  maxWeight = " + maxWeight + "\n");
    sb.append("  learningRate = " + learningRate + "\n");
    sb.append("  momentum = " + momentum + "\n");
    sb.append("  errorCriterion = " + errorCriterion + "\n");
    sb.append("  numEpochs = " + numEpochs + "\n");
    sb.append("  trainType = " + trainType + "\n");
    sb.append("  numInputs = " + numInputs + "\n");
    sb.append("  numOutputs = " + numOutputs + "\n");
    sb.append("  numHidden = " + numHidden + "\n");
    sb.append("  numHidLayers = " + numHidLayers + "\n");
    sb.append("}\n");
    return sb.toString();
  }
}
